#include <LiquidCrystal_I2C.h>
#include <IRremote.h>
#include <Keypad.h>
#include <Servo.h>
#include <Arduino.h>
#include <SPI.h>
#include <MFRC522.h>

#define   passlen   5
#define   SERVO     17
#define   BUZZER    15
#define   IR        16
#define   ON        16718055
#define   OFF       16730805
#define   RST_PIN   9         
#define   SS_PIN    10
#define   LED       14

IRrecv irrecv(IR);
decode_results results;

LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);

const byte ROWS = 4;
const byte COLS = 3;
char keys[ROWS][COLS] = {
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
};
byte rowPins[ROWS] = {2, 3, 4, 5};
byte colPins[COLS] = {6, 7, 8};

Servo servo;
int angle = 10;

Keypad keypad = Keypad(makeKeymap(keys), rowPins, colPins, ROWS, COLS);

char data[passlen];
char pass[passlen] = "1579";
int counter = 0;

MFRC522 mfrc522(SS_PIN, RST_PIN);

int lastEvent = 0;

void activateDoor() {
  lastEvent = 0;
  
  if (angle < 60) {
    lcd.clear();
    lcd.setCursor(1, 0);
    lcd.print("Door unlocked!");
    digitalWrite(LED, LOW);
    pinMode(BUZZER, OUTPUT);
    digitalWrite(BUZZER, LOW);
    delay(500);
    digitalWrite(BUZZER, HIGH);
    pinMode(BUZZER, INPUT);
    while(angle < 60) {
      angle++;
      servo.write(angle);
      delay(15);
    }
    memset(data, 0, 4 * sizeof(char)); 
    delay(2000);
    lcd.clear();
    digitalWrite(LED, HIGH);
  } else {
    lcd.clear();
    digitalWrite(LED, LOW);
    lcd.setCursor(2, 0);
    lcd.print("Door already");
    lcd.setCursor(3, 1);
    lcd.print("unlocked!");
    pinMode(BUZZER, OUTPUT);
    digitalWrite(BUZZER, LOW);
    delay(100);
    digitalWrite(BUZZER, HIGH);
    delay(100);
    digitalWrite(BUZZER, LOW);
    delay(300);
    pinMode(BUZZER, INPUT);
    delay(2000);
    lcd.clear();
    digitalWrite(LED, HIGH);
  }
}

void closeDoor() {
  lastEvent = 0;
 
  if (angle > 10) {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Door locked!");
    digitalWrite(LED, LOW);
    pinMode(BUZZER, OUTPUT);
    digitalWrite(BUZZER, LOW);
    delay(50);
    digitalWrite(BUZZER, HIGH);
    delay(100);
    digitalWrite(BUZZER, LOW);
    delay(100);
    pinMode(BUZZER, INPUT);
    while(angle > 10) {
      angle--;
      servo.write(angle);
      delay(15);
    }
    lcd.clear();
    digitalWrite(LED, HIGH);
  } else {
      lcd.clear();
      lcd.setCursor(2, 0);
      lcd.print("Door already");
      lcd.setCursor(4, 1);
      lcd.print("locked!");
      digitalWrite(LED, LOW);
      pinMode(BUZZER, OUTPUT);
      digitalWrite(BUZZER, LOW);
      delay(100);
      digitalWrite(BUZZER, HIGH);
      delay(100);
      digitalWrite(BUZZER, LOW);
      delay(300);
      pinMode(BUZZER, INPUT);
      delay(2000);
      lcd.clear();
      digitalWrite(LED, HIGH);
  }
}

void setup() {
  lcd.begin(16, 2);
  lcd.clear();

  irrecv.enableIRIn();
  
  servo.attach(SERVO);
  servo.write(angle);

  pinMode(BUZZER, INPUT);

  SPI.begin();    
  mfrc522.PCD_Init();

  pinMode(LED, OUTPUT);
  digitalWrite(LED, HIGH);
}

void loop() {
  lcd.setCursor(1, 0);
  lcd.print("Enter password:");
  lcd.setCursor(6, 1);

  lastEvent ++;
 
  if (mfrc522.PICC_IsNewCardPresent() && mfrc522.PICC_ReadCardSerial()) {
    String content= "";
    byte letter;
    for (byte i = 0; i < mfrc522.uid.size; i++) 
    {
       content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
       content.concat(String(mfrc522.uid.uidByte[i], HEX));
    }
    content.toUpperCase();
    if (content.substring(1) == "53 75 0E 15" || content.substring(1) == "6C E6 17 4A") {
      activateDoor();
    } 
  }

  if(irrecv.decode(&results)) {
    if(results.value == ON) {
      activateDoor();
    } else if(results.value == OFF) {
      closeDoor();
    }
    irrecv.resume();
  }
  
  
  char key = keypad.getKey();
  if (key) {
    data[counter] = key;
    lcd.setCursor(counter + 6, 1);
    lcd.print(key);
    counter++;
    if (counter == 4) {
      delay(500);
      lcd.clear();
      counter = 0;
      if (strcmp(data, pass) == 0) {
        activateDoor();
      } else {
        lastEvent = 0;
     
        digitalWrite(LED, LOW);
        lcd.setCursor(0, 0);
        pinMode(BUZZER, OUTPUT);
        digitalWrite(BUZZER, LOW);
        delay(300);
        digitalWrite(BUZZER, HIGH);
        lcd.print("Wrong password!");
        delay(300);
        pinMode(BUZZER, INPUT);
        memset(data, 0, 4 * sizeof(char)); 
        delay(2000);
        lcd.clear();
        digitalWrite(LED, HIGH);
      }
    }
  }

  if(lastEvent == 200) {
    lastEvent = 0;
    if(angle > 10) {
      closeDoor();
    }
  }
} 
